@@ -11,9 +11,16 @@ mainly 2 powerful packages sqlalchemy and pandas.
 
 Main Features
 -------------
-Here are just a few of the things that sql-executor does well:
+Here are just a few of the things that sql-executor can do:
 
-  - first
+  - to be written...
 """
 
-from .sql_executor import SqlExecutor
+import logging
+
+
+from sqldbclient.sql_executor import SqlExecutor, SqlExecutorConf
+from sqldbclient.sql_engine_factory import sql_engine_factory
+
+
+logging.getLogger(__name__)
